#include "StdAfx.h"
#include "ContinusFormula.h"

CContinusFormula::CContinusFormula( void )
{
   pi = 4.0*atan( 1.0 );
}
CContinusFormula::~CContinusFormula( void )
{
}
int CContinusFormula::ContinusFormula_SetVariables  ( const double *formulavalues, const int nmodelparameter )
//----------------------------------------------------------------------
// --- initalize parameter from argument list
//----------------------------------------------------------------------
{
   double value;
   int    j=0;
//
   Nmodelparameter = nmodelparameter;
//
// --- model variables stiffness
//
   cx0      = formulavalues[j++];
   cx1      = formulavalues[j++];  if ( version > 201 ) cx2 = formulavalues[j++];
   cy0      = formulavalues[j++];
   cy1      = formulavalues[j++];  if ( version > 201 ) cy2 = formulavalues[j++];
//
// --- model variables offsets
//
   sx0      = formulavalues[j++];
   sy0      = formulavalues[j++];
   soff     = formulavalues[j++];
//
// --- model variables friction, force positive value
//
   mue0     = fabs( formulavalues[j++] );
   value    = formulavalues[j++];
   value    = formulavalues[j++];
   mue1xp   = fabs( formulavalues[j++] );
   cmuexp   = fabs( formulavalues[j++] );
   mue1xn   = fabs( formulavalues[j++] );
   cmuexn   = fabs( formulavalues[j++] );
   mue1yp   = fabs( formulavalues[j++] );
   cmueyp   = fabs( formulavalues[j++] );
   mue1yn   = fabs( formulavalues[j++] );
   cmueyn   = fabs( formulavalues[j++] );
   mueFz    = formulavalues[j++];
   vslide0  = formulavalues[j++];
//
// --- model variables other
//
   pin      = formulavalues[j++];
   Fzbar    = formulavalues[j++];
   MzScaleP = formulavalues[j++];
   MzScaleN = formulavalues[j++];
//
// --- model variables contact
//
   cR       = formulavalues[j++];
   R        = formulavalues[j++];

   return ( 0 );
}
int CContinusFormula::ContinusFormula_SetVariables  ( double *formulavalues, const double *formuladesigns, const int *isdesvar )
//----------------------------------------------------------------------
// --- initalize parameter from argument list
//----------------------------------------------------------------------
{
   double value;
   int i,j=0,l=0;
//
// --- refresh model variables by new design
//
   for ( i=0; i<Nmodelparameter; i++ )
   {
      if ( isdesvar[i] )
      {
         formulavalues[i] = formuladesigns[l];
         l++;
      }
   }
   cx0      = formulavalues[j++];
   cx1      = formulavalues[j++];  if ( version > 201 ) cx2 = formulavalues[j++];
   cy0      = formulavalues[j++];
   cy1      = formulavalues[j++];  if ( version > 201 ) cy2 = formulavalues[j++];
//
// --- model variables offsets
//
   sx0      = formulavalues[j++];
   sy0      = formulavalues[j++];
   soff     = formulavalues[j++];
//
// --- model variables friction, force positive value
//
   mue0     = fabs( formulavalues[j++] );
   value    = formulavalues[j++];
   value    = formulavalues[j++];
   mue1xp   = fabs( formulavalues[j++] );
   cmuexp   = fabs( formulavalues[j++] );
   mue1xn   = fabs( formulavalues[j++] );
   cmuexn   = fabs( formulavalues[j++] );
   mue1yp   = fabs( formulavalues[j++] );
   cmueyp   = fabs( formulavalues[j++] );
   mue1yn   = fabs( formulavalues[j++] );
   cmueyn   = fabs( formulavalues[j++] );
   mueFz    = formulavalues[j++];
   vslide0  = formulavalues[j++];
//
// --- model variables other
//
   pin      = formulavalues[j++];
   Fzbar     = formulavalues[j++];
   MzScaleP = formulavalues[j++];
   MzScaleN = formulavalues[j++];
//
// --- model variables contact
//
   cR       = formulavalues[j++];
   R        = formulavalues[j++];

   return ( 0 );
}
void CContinusFormula::ContinusFormula_Evaluate   ( const double *X, double *Y )
//------------------------------------------------------------------------------
// --- evaluate formula
//------------------------------------------------------------------------------
{
   double sx,sy,VT,term1,term2,term3;
   double xi,eta,resslip;
   double mue1x,cmuex,mue1y,cmuey,muex,muey;
   double sig,kappa,cx,cy;
   double muex1Fz,muey1Fz,cmuexFz,cmueyFz;
//
// --- assign local input variable
//
   sx = X[0];
   sy = X[1];
   Fz = X[3];
   VT = X[4];
//
// --- nothing to do
//
   if ( Fz == 0 )
   {
      Y[0] = 0.0;
      Y[1] = 0.0;
      Y[2] = 0.0;
      return;
   }
//
// --- assign conversions and offsets
//
   cx = cx0 + cx1*( 1.0 + ( Fzbar-Fz ) );
   cy = cy0 + cy1*( 1.0 + ( Fzbar-Fz ) );

   sy = sy + sy0 + ( Fzbar-Fz )*soff;
   sx = sx + sx0 + ( Fzbar-Fz )*soff;
   sy  = tan( sy * pi/180 );
//
// --- assign friction values
//
   if ( sx > 0.0 )
   {
      mue1x = mue1xp;
      cmuex = cmuexp;
   }
   else
   {
      mue1x = mue1xn;
      cmuex = cmuexn;
   }

   if ( sy > 0.0 )
   {
      mue1y = mue1yp;
      cmuey = cmueyp;
   }
   else
   {
      mue1y = mue1yn;
      cmuey = cmueyn;
   }

   mue0Fz  = mue0* ( 1.0 + mueFz*( Fzbar-Fz ) );
   muex1Fz = mue1x*( 1.0 + mueFz*( Fzbar-Fz ) );
   muey1Fz = mue1y*( 1.0 + mueFz*( Fzbar-Fz ) );
   cmuexFz = cmuex*( 1.0 + mueFz*( Fzbar-Fz ) );
   cmueyFz = cmuey*( 1.0 + mueFz*( Fzbar-Fz ) );
//
// --- temp variables
//
   resslip = sqrt( sx*sx + sy*sy );

   if ( ( fabs( sy ) < 1.0e-09 ) && ( fabs( sx ) < 1.0e-09 ) ) resslip = 1.0;

   eta   = sy/resslip;
   xi    = sx/resslip;
   muex  = mue0Fz + ( mue0Fz-muex1Fz )*( exp( -abs( VT*resslip )/cmuexFz )-1.0 );
   muey  = mue0Fz + ( mue0Fz-muey1Fz )*( exp( -abs( VT*resslip )/cmueyFz )-1.0 );
   h     = sqrt( R*R-( R-Fz/cR )*( R-Fz/cR ) );
   b     = Fz/( 4.0*pin*h );
   sig   = 3.0*pin/2.0;
   kappa = h*h*sqrt( cx*cx*sx*sx +cy*cy*sy*sy )/( mue0Fz*sig ) - h;

   if ( kappa > h ) kappa = h;

   term1 = ( kappa-h )*( kappa-h );
   term2 = 2.0*h-kappa;
   term3 = ( kappa+h )*( kappa+h );

   Y[0] = b*cx*sx*term1                   + 2.0*b*xi *muex*sig*term2*term3/( 3.0*h*h );
   Y[1] = b*cy*sy*term1                   + 2.0*b*eta*muey*sig*term2*term3/( 3.0*h*h );
   Y[2] = b*cy*sy*term1*( 2.0*kappa+h )/3.0   - b*eta*muey*sig*term1*term3/( 2.0*h*h );

   if ( sy > 0.0 ) Y [2] = -Y [2]*MzScaleP;
   else            Y [2] = -Y [2]*MzScaleN;
}
void CContinusFormula::ContinusFormula_Terminate  ( )
//----------------------------------------------------------------------
// --- terminate formula
//----------------------------------------------------------------------
{
}