#include "StdAfx.h"
#include "DiscreteFormula.h"

CDiscreteFormula::CDiscreteFormula( void )
{
   pi = 4.0*atan( 1.0 );
	vslstep = 3000.0;
	xygamma =    2.0;
	
	dataxpow1 = 2.0;
	dataxpow2 = 2.0;
	dataypow1 = 7.0;
	dataypow2 = 2.0;
	dataypow3 = 5.0;

	FxTurnOn   = 0.02;
	GammaCoeff = 1.00;
	GammaB     = 0.00;

	dataexchange          = 0;
	dataexchangeallocated = 0;
	IterateFy             = 1;

	myDataPointerContainer = new CDataPointerContainer;
}
CDiscreteFormula::~CDiscreteFormula( void )
{
}
int CDiscreteFormula::DiscreteFormula_SetExVariables ( const int j, const int i, const double value)
//--------------------------------------------------------------------------------------------------------------
// --- initalize parameter for extended contact area description
//--------------------------------------------------------------------------------------------------------------
{
	int ierr=0;

	switch (j)
	{
		case(1):											 
			switch (i)
			{
				case(1):	{Parameter_LR.av  = value; break;}
				case(2):	{Parameter_LR.pv  = value; break;}
				case(3):	{Parameter_LR.kz  = value; break;}
				case(4):	{Parameter_LR.ky  = value; break;}
				case(5):	{Parameter_LR.kc  = value; break;}
				case(6):	{Parameter_LR.ac  = value; break;}
				case(7):	{Parameter_LR.az  = value; break;}
				case(8):	{Parameter_LR.p0  = value; break;}
				case(9):	{Parameter_LR.R0  = value; break;}
				default: {ierr = 1; break;}
			}
			break;
		case(2):
			switch (i)
			{
				case(1):	{Par.R_y         = value; break;}
				case(2):	{Par.p1          = value; break;}
				case(3):	{Par.p2          = value; break;}
				case(4):	{Par.gamma_sin   = value; break;}
				case(5):	{Par.gamma_cos   = value; break;}
				case(6):	{Par.gamma_comp  = value; break;}
				case(7):	{Par.offset      = value; break;}
				case(8):	{Par.ny          = value; break;}
				case(9):	{Par.ky_stat     = value; break;}
				default: {ierr = 2; break;}
			}
			break;
		default: {return (ierr);}
	}
	return (ierr);
}
double CDiscreteFormula::DiscreteFormula_GetExVariables ( const int j, const int i)
//--------------------------------------------------------------------------------------------------------------
// --- initalize parameter for extended contact area description
//--------------------------------------------------------------------------------------------------------------
{
	int ierr=0;

	switch (j)
	{
		case(1):											 
			switch (i)
			{
				case(1):	{return(Parameter_LR.av);}
				case(2):	{return(Parameter_LR.pv);}
				case(3):	{return(Parameter_LR.kz);}
				case(4):	{return(Parameter_LR.ky);}
				case(5):	{return(Parameter_LR.kc);}
				case(6):	{return(Parameter_LR.ac);}
				case(7):	{return(Parameter_LR.az);}
				case(8):	{return(Parameter_LR.p0);}
				case(9):	{return(Parameter_LR.R0);}
				default: {ierr = 1; break;}
			}
			break;
		case(2):
			switch (i)
			{
				case(1):	{return(Par.R_y);}
				case(2):	{return(Par.p1);}
				case(3):	{return(Par.p2);}
				case(4):	{return(Par.gamma_sin);}
				case(5):	{return(Par.gamma_cos);}
				case(6):	{return(Par.gamma_comp);}
				case(7):	{return(Par.offset);}
				case(8):	{return(Par.ny);}
				case(9):	{return(Par.ky_stat);}
				default: {ierr = 2; break;}
			}
			break;
		default: {return (ierr);}
	}
	return (ierr);
}
int CDiscreteFormula::DiscreteFormula_SetVariables  ( const double *formulavalues, const int nmodelparameter )
//--------------------------------------------------------------------------------------------------------------
// --- initalize parameter from argument list
//--------------------------------------------------------------------------------------------------------------
{
	double value,value1;
   int    j=0,tyre;

	Nmodelparameter = nmodelparameter;
//
// --- model variables stiffness
//
   cx0      = formulavalues[j++];
	cx1      = formulavalues[j++]; if ( version == 202 ) {cx2 = formulavalues[j++];}
   cy0      = formulavalues[j++];
	cy1      = formulavalues[j++]; if ( version == 202 ) {cy2 = formulavalues[j++];}
//
// --- model variables offsets
//
   sx0      = formulavalues[j++]; 
   sy0      = formulavalues[j++]; 
   soff     = formulavalues[j++];  
//
// --- model variables friction, force positive value
//
   mue0     = fabs( formulavalues[j++] );  
   muexy    = formulavalues[j++];  
   Vmuexy   = formulavalues[j++]; 
   mue1xp   = fabs( formulavalues[j++] );
   cmuexp   = fabs( formulavalues[j++] );
   mue1xn   = fabs( formulavalues[j++] );
   cmuexn   = fabs( formulavalues[j++] );
   mue1yp   = fabs( formulavalues[j++] );
   cmueyp   = fabs( formulavalues[j++] );
   mue1yn   = fabs( formulavalues[j++] );
   cmueyn   = fabs( formulavalues[j++] ); 
   mueFz    = formulavalues[j++];
   vslide0  = formulavalues[j++]; 
//
// --- model variables other
//
   pin      = formulavalues[j++]; 
   Fzbar    = formulavalues[j++]; 
   StiffSca = formulavalues[j++]; 
   FrictSca = formulavalues[j++];  
//
// --- model variables contact
//
   cR       = formulavalues[j++];
   R        = formulavalues[j++];
   Ry       = formulavalues[j++]; 

   ni       = ( int ) ceil ( formulavalues[j++] );
   nj       = ( int ) ceil ( formulavalues[j++] ); 
//
// --- check ny to be even
//
	value1 = formulavalues[j++];  
	value = fmod(value1,2.0); 
	if ( value != 0 ) ny = (int)(value1 - value);
	else              ny = (int) value1;

   sliding  = ( int ) ceil ( formulavalues[j++] );
   pressure = ( int ) ceil ( formulavalues[j++] );
   draw3d   = ( int ) ceil ( formulavalues[j++] ); 

	if ( version >= 203 )
	{
		vslstep   = formulavalues[j++]; 
		dataxpow1 = formulavalues[j++];	
		dataxpow2 = formulavalues[j++];	
		dataypow1 = formulavalues[j++]; 
		dataypow2 = formulavalues[j++];	
		dataypow3 = formulavalues[j++];	
		xygamma   = formulavalues[j++]; 
		IterateFy = (int) formulavalues[j++];  
		FxTurnOn  = formulavalues[j++];
	}

	if ( version >= 204 )
	{
		GammaCoeff = formulavalues[j++];
		GammaB     = formulavalues[j++]; 
	}

   islide = Nslide;
	tyre   = 1;

   if ( ( bmaxj    = ( double * ) calloc (   ni+1,sizeof( double ) ) ) == NULL ) return ( 3 );
   if ( ( bminj    = ( double * ) calloc (   ni+1,sizeof( double ) ) ) == NULL ) return ( 4 );
   if ( ( hi       = ( double * ) calloc (   nj+1,sizeof( double ) ) ) == NULL ) return ( 5 );
	if ( ( muslide  = ( double * ) calloc ( islide,sizeof( double ) ) ) == NULL ) return ( 6 );
   if ( ( vslide   = ( double * ) calloc ( islide,sizeof( double ) ) ) == NULL ) return ( 7 );

   Fzscale = alloc_mat ( ni+1,nj+1,0.0 );
   DiscreteFormula_Set_Friction_FB ();
//
// --- initalise contact patch data exchange class 
//
	if ( dataexchange && !dataexchangeallocated )
	{
		if ( ( mypatchoutput.fricpower = ( double * ) calloc ( ni*nj, sizeof( double ) ) ) == NULL ) return ( 8 );
		if ( ( mypatchoutput.damppower = ( double * ) calloc ( ni*nj, sizeof( double ) ) ) == NULL ) return ( 9 );

		mypatchoutput.nx = ni;
		mypatchoutput.my = nj;

		myDataPointerContainer->Set_DataPointer_out(&mypatchoutput,tyre);
		dataexchangeallocated = 1;
	}

   return ( 0 );
}
int CDiscreteFormula::DiscreteFormula_SetVariables  ( double *formulavalues, const double *formuladesigns, const int *isdesvar )
//------------------------------------------------------------------------------------------------------------------------------
// --- initalize parameter from argument list
//------------------------------------------------------------------------------------------------------------------------------ 
{
   int    i,j=0,l=0,tyre;
//
// --- refresh model variables by new design
//
   for ( i=0; i<Nmodelparameter; i++ )
   {
      if ( isdesvar[i] )
      {
         formulavalues[i] = formuladesigns[l];
         l++;
      }
   }
//
// --- model variables stiffness
//
   cx0      = formulavalues[j++];
   cx1      = formulavalues[j++];  if ( version == 202 ) cx2 = formulavalues[j++];
   cy0      = formulavalues[j++];
   cy1      = formulavalues[j++];  if ( version == 202 ) cy2 = formulavalues[j++];
//
// --- model variables offsets
//
   sx0      = formulavalues[j++];
   sy0      = formulavalues[j++];
   soff     = formulavalues[j++];
//
// --- model variables friction, force positive value
//
   mue0     = fabs( formulavalues[j++] );
   muexy    = formulavalues[j++];
   Vmuexy   = formulavalues[j++];
   mue1xp   = fabs( formulavalues[j++] );
   cmuexp   = fabs( formulavalues[j++] );
   mue1xn   = fabs( formulavalues[j++] );
   cmuexn   = fabs( formulavalues[j++] );
   mue1yp   = fabs( formulavalues[j++] );
   cmueyp   = fabs( formulavalues[j++] );
   mue1yn   = fabs( formulavalues[j++] );
   cmueyn   = fabs( formulavalues[j++] );
   mueFz    = formulavalues[j++];
   vslide0  = formulavalues[j++];
//
// --- model variables other
//
   pin      = formulavalues[j++];
   Fzbar    = formulavalues[j++];
   StiffSca = formulavalues[j++];
   FrictSca = formulavalues[j++];
//
// --- model variables contact
//
   cR       = formulavalues[j++];
   R        = formulavalues[j++];
   Ry       = formulavalues[j++];

   ni       = ( int ) ceil ( formulavalues[j++] );
   nj       = ( int ) ceil ( formulavalues[j++] );
   ny       = ( int ) ceil ( formulavalues[j++] );
   sliding  = ( int ) ceil ( formulavalues[j++] );
   pressure = ( int ) ceil ( formulavalues[j++] );
   draw3d   = ( int ) ceil ( formulavalues[j++] );

	if ( version >= 203 )
	{
		vslstep   = formulavalues[j++];
		dataxpow1 = formulavalues[j++];
		dataxpow2 = formulavalues[j++];
		dataypow1 = formulavalues[j++];
		dataypow2 = formulavalues[j++];
		dataypow3 = formulavalues[j++];
		xygamma   = formulavalues[j++];
		IterateFy = ( int ) ( formulavalues[j++] );
		FxTurnOn  = formulavalues[j++];
	}

	if ( version >= 204 )
	{
		GammaCoeff = formulavalues[j++];
		GammaB     = formulavalues[j++];
	}

   DiscreteFormula_Set_Friction_FB ();
	tyre = 1;
//
// --- initalise contact patch data exchange class 
//	
	if ( !dataexchangeallocated )
	{
		if ( ( mypatchoutput.fricpower = ( double * ) calloc ( ni*nj, sizeof( double ) ) ) == NULL ) return ( 6 );
		if ( ( mypatchoutput.damppower = ( double * ) calloc ( ni*nj, sizeof( double ) ) ) == NULL ) return ( 7 );

		mypatchoutput.nx = ni;
		mypatchoutput.my = nj;

		myDataPointerContainer->Set_DataPointer_out(&mypatchoutput,tyre);
		dataexchangeallocated = 1;
	}

   return ( 0 );
}
void CDiscreteFormula::DiscreteFormula_GetVariables  ( double *formulavalues )
//------------------------------------------------------------------------------------------------------------------------------
// --- initalize parameter from argument list
//------------------------------------------------------------------------------------------------------------------------------ 
{
   int j=0;
//
// --- model variables stiffness
//
   formulavalues[j++] = cx0;
   formulavalues[j++] = cx1;  
	if ( version == 202 ) formulavalues[j++] = cx2;
   formulavalues[j++] = cy0;
   formulavalues[j++] = cy1;  
	if ( version == 202 ) formulavalues[j++] = cy2;
//
// --- model variables offsets
//
   formulavalues[j++] = sx0;
   formulavalues[j++] = sy0;
   formulavalues[j++] = soff;
//
// --- model variables friction, force positive value
//
   formulavalues[j++] = mue0;   
   formulavalues[j++] = muexy;  
   formulavalues[j++] = Vmuexy;  
   formulavalues[j++] = fabs( mue1xp); 
   formulavalues[j++] = fabs( cmuexp); 
   formulavalues[j++] = fabs( mue1xn); 
   formulavalues[j++] = fabs( cmuexn); 
   formulavalues[j++] = fabs( mue1yp); 
   formulavalues[j++] = fabs( cmueyp); 
   formulavalues[j++] = fabs( mue1yn); 
   formulavalues[j++] = fabs( cmueyn);  
   formulavalues[j++] = mueFz;    
   formulavalues[j++] = vslide0;   
//
// --- model variables other
//
   formulavalues[j++] = pin;     
   formulavalues[j++] = Fzbar;   
   formulavalues[j++] = StiffSca;
   formulavalues[j++] = FrictSca; 
//
// --- model variables contact
//
   formulavalues[j++] = cR;
   formulavalues[j++] = R;
   formulavalues[j++] = Ry;

   formulavalues[j++] = ( int ) ( ni );
   formulavalues[j++] = ( int ) ( nj );
   formulavalues[j++] = ( int ) ( ny );
   formulavalues[j++] = ( int ) ( sliding );
   formulavalues[j++] = ( int ) ( pressure );
   formulavalues[j++] = ( int ) ( draw3d );

	if ( version >= 203 )
	{
		formulavalues[j++] = vslstep;
		formulavalues[j++] = dataxpow1;
		formulavalues[j++] = dataxpow2;
		formulavalues[j++] = dataypow1;
		formulavalues[j++] = dataypow2;
		formulavalues[j++] = dataypow3;
		formulavalues[j++] = xygamma;
		formulavalues[j++] = ( int ) ( IterateFy  );
		formulavalues[j++] = FxTurnOn;
	}
	if ( version >= 204 )
	{
		formulavalues[j++] = GammaCoeff;
		formulavalues[j++] = GammaB;
	}
}
void CDiscreteFormula::DiscreteFormula_Set_Friction_FB ()
//----------------------------------------------------------------------
// --- data of picewise linear friction function
//----------------------------------------------------------------------
{
//
// --- used design variable name of e-friction function
//
   vslide [0] = vslide0;
   vslide [1] = vslstep + vslide[0];
   vslide [2] = vslstep + vslide[1];
   vslide [3] = vslstep + vslide[2];
   vslide [4] = vslstep + vslide[3];
   vslide [5] = vslstep + vslide[4];
									 
   muslide [0] = fabs( mue1xp );
   muslide [1] = fabs( cmuexp );
   muslide [2] = fabs( mue1xn );
   muslide [3] = fabs( cmuexn );
   muslide [4] = fabs( mue1yp );
   muslide [5] = fabs( cmueyp );

   frica0 = mue1yn;
   frica1 = cmueyn;
}
void CDiscreteFormula::DiscreteFormula_Initialize ( )
//----------------------------------------------------------------------
// --- initalize parameter from file
//----------------------------------------------------------------------
{
   if ( draw3d == 1 && dataexchange )
   {
      DiscreteOutput = fopen ( "DiscreteOutput.dat","w" );
      rewind ( DiscreteOutput );
   }
}
void CDiscreteFormula::DiscreteFormula_Evaluate ( const double *X, double *Y )
//----------------------------------------------------------------------
// --- call to evaluate formula on regular grid
//----------------------------------------------------------------------
{
	int i;
//
// --- lateral force interation ?
//
	if ( IterateFy > 0 ) draw3d  = 0;
	gammaFy = 0.0;
	Y[1]    = 0.0;
	DiscreteFormula_Compute ( X, Y );

	if ( IterateFy == 0 ) return;
//
// --- lateral force interation !
//
	for ( i=0; i<IterateFy; i++ )
	{
		if ( i == IterateFy-1 ) draw3d  = 1;
		gammaFy = -GammaB*Y[1];
		DiscreteFormula_Compute ( X, Y );
	}
}
void CDiscreteFormula::DiscreteFormula_Compute ( const double *X, double *Y )
//-------------------------------------------------------------------------- 
// --- evaluate formula on regular grid
//--------------------------------------------------------------------------
{
   int    i,j;
   double cxq,cyq;
   double xi,yj,hvj;
   double value1,value2,value3,value4;
   double value,fvalue1,fvalue2;
//
// --- assign local input values
//
   sx = X[0];
   sy = X[1];
   st = X[2];
   Fz = X[3];
   VT = X[4];
//
//--- zero wheel load leads to zero forces
//
	if ( Fz < 1.0E-05 )
	{
		Y [0] = 0.0;
		Y [1] = 0.0;
		Y [2] = 0.0;
		Y [3] = 0.0;
		Y [4] = 0.0;
		Y [5] = 0.0;
		Y [6] = 0.0;
		return;
	}
//
//--- sticking and creep force components
//
	fxsl   = 0.0;
	fysl   = 0.0;
	fxst   = 0.0;
	fyst   = 0.0;
	Area   = 0.0;
	gamma  = st * pi/ 180.0;
   valij  = ( ( double ) ni ) * ( ( double ) nj );
   value1 = ( double ) (nj*nj);
   value2 = ( double ) (ni*ni);
   value3 = value2*value2;
//
// --- check contact area
//
	if ( version >= 204 ) DiscreteFormula_Area_Lookup_Ex(X[3],Y[1],VT);
	else                  DiscreteFormula_Area_Lookup();

   dxf = 2.0*h / ( ( double ) ni );
   dyf = 2.0*b / ( ( double ) nj );
	dA  = dxf*dyf;
//
// --- local variables that change with Fz
//
	if ( version != 202 )
	{
		cxq = ( cx0 + cx1*( 1.0 - Fz/Fzbar ) )*dA;
		cyq = ( cy0 + cy1*( 1.0 - Fz/Fzbar ) )*dA;
	}
	else
	{
		value4 = Fzbar/Fz - 1.0;
		cxq    = ( cx0 + cx1*value4 + cx2*value4*value4 )*dA;
		cyq    = ( cy0 + cy1*value4 + cy2*value4*value4 )*dA;
	}

	cxq = cxq*StiffSca;
	cyq = cyq*StiffSca;
//
// --- normal stress magnitudes
//
   if ( pressure != 4 )
	{
		sigma0 =   3.0*Fz/2.0;
		sigma1 =  45.0*Fz*value1*value3/( ( 48.0*value1-12.0 )*value3+( 40.0*value1-10.0 )*value2-28.0*value1+7.0 );
		sigma2 = 225.0*Fz*value1*value1*value3/
					( ( 216.0*value1*value1 + 300.0*value1 - 336.0 )*value3 +
					  ( 180.0*value1*value1 + 250.0*value1 - 280.0 )*value2 -
						 126.0*value1*value1 - 175.0*value1 + 196.0 );
	}

	if ( version != 202 )
	{
	  sy    = sy + sy0 + ( 1.0 + ( Fzbar-Fz ) )*soff;
	  sx    = sx + sx0 + ( 1.0 + ( Fzbar-Fz ) )*soff;
	}
	else
	{
	  sy    = sy + sy0 + ( ( Fz/Fzbar ) - 1.0 )*soff;
	  sx    = sx + sx0 + ( ( Fz/Fzbar ) - 1.0 )*soff;
	}
   sy = tan( sy * pi/ 180.0 );
   st = sin( st * pi/( 180.0 * R ) ) + gammaFy/ R;
//
// --- analyse contact area
//
   if ( pressure == 4 )
   {
      Fzmag = 1.0;
		if ( version >= 204 ) DiscreteFormula_Area_Ex(X[3],Y[1]);
		else                  DiscreteFormula_Area();
      if ( sigma3 < 1.0e-10 ) Fzmag = 0.0;
		else                    Fzmag = Fz/sigma3;
		dA = dA * ( (double) valij ) / ninside;
   }
//
// --- preset contact forces
//
   Y[0] = 0.0;
   Y[1] = 0.0;
   Y[2] = 0.0;
//
// --- steady state transport equations
//
   if ( VT < 0.0 )
   {
      for ( j=0; j<=nj; j++ )
      {
         yj = b - dyf * ( double ) j;
         for ( i=0; i<=ni; i++ )
         {
				hvj = hi[j];
				if ( hvj > 0.0 )
				{
					xi     = h - dxf * ( double ) i;
					value1 = xi/hvj;
					u      = ( 1.0 - value1 )*hvj*( sx - FxTurnOn*yj*st );
					v      = ( 1.0 - value1 )*hvj*sy + ( 1.0 - value1*value1 )*hvj*hvj*st*0.5*GammaCoeff;
					fx     = cxq * u;
					fy     = cyq * v;
					vsx    = ( sx + FxTurnOn*yj*st )*fabs( VT );
					vsy    = ( sy +          xi*st )*fabs( VT );
//		
// --- normal force
//
					if ( pressure < 4 ) fnij = DiscreteFormula_Pressure ( xi,yj );
					else                fnij = DiscreteFormula_GetFz    ( xi,yj,i,j );
					if ( fnij < 1.0e-12 ) 
					{
						fx    = 0.0;
						fy    = 0.0;
						value = 0.0;
						goto L1;
					}
//
// --- slip depending friction
//
					if ( sliding  < 4 ) value = DiscreteFormula_Friction ();
					else                value = DiscreteFormula_Friction_FB ();
//
// --- look inside contact patch
//
L1:				if ( draw3d && dataexchange )
					{
						if (  fnij > 0.0 && value > 0.0 ) fvalue1 = (fx*vsx + fy*vsy);
						else                              fvalue1 = 0.0;
						if (  fnij > 0.0 ) fvalue2 = vsx*vsx + vsy*vsy;
						else               fvalue2 = 0.0;
						mypatchoutput.fricpower[i+j*nj] = fvalue1;
						mypatchoutput.damppower[i+j*nj] = fvalue2;
						if ( draw3d ) fprintf ( DiscreteOutput," %+12.6e %+12.6e %+12.6e %+12.6e %+12.6e %+12.6e %+12.6e %+12.6e %+12.6e %+12.6e\n",xi,yj,u,v,fnij/dA,fx/dA,fy/dA,fvalue1,fvalue2,value);
					}
//
// --- force and moment
//
   				Y [0] = Y[0] + fx;
					Y [1] = Y[1] + fy;
					Y [2] = Y[2] + fy*xi - fx*yj;
				}
         }
         if ( draw3d && dataexchange ) fprintf ( DiscreteOutput,"\n" );
      }
   }
   else
   {
      for ( j=0; j<=nj; j++ )
      {
         yj = b - dyf * ( double ) j;
         for ( i=ni; i>=0; i-- )
         {
				hvj = hi[j];
				if ( hvj > 0.0 )
				{
					xi     = h - dxf * ( double ) i;
					value1 = xi/hvj;
					u      = ( value1 + 1.0 )*hvj*( sx - FxTurnOn*yj*st );
					v      = ( value1 + 1.0 )*hvj*sy + ( 1.0 - value1*value1 )*hvj*hvj*st*0.5*GammaCoeff;
					fx     = cxq * u;
					fy     = cyq * v;
					vsx    = ( sx - FxTurnOn*yj*st )*fabs( VT );
					vsy    = ( sy +          xi*st )*fabs( VT );
//
// --- normal force
//
					if ( pressure < 4 ) fnij = DiscreteFormula_Pressure ( xi,yj );
					else                fnij = DiscreteFormula_GetFz    ( xi,yj,i,j );
					if ( fnij < 1.0e-12 ) 
					{
						fx    = 0.0;
						fy    = 0.0;
						value = 0.0;
						goto L2;
					}
//
// --- sticking and sliding, slip depending friction
//
					if ( sliding  < 4 ) value = DiscreteFormula_Friction ();
					else                value = DiscreteFormula_Friction_FB ();
//
// --- look inside contact patch
//
L2:				if ( draw3d && dataexchange )
					{
						if (  fnij > 0.0 && value > 0.0 ) fvalue1 = (fx*vsx + fy*vsy);
 						else                              fvalue1 = 0.0;
						if (  fnij > 0.0 ) fvalue2 = vsx*vsx + vsy*vsy;
 						else               fvalue2 = 0.0;
						mypatchoutput.fricpower[i+j*nj] =         fvalue1;
						mypatchoutput.damppower[i+j*nj] = 7.0e-04*fvalue2;
						if ( draw3d ) fprintf ( DiscreteOutput," %+12.6e %+12.6e %+12.6e %+12.6e %+12.6e %+12.6e %+12.6e %+12.6e %+12.6e %+12.6e\n",xi,yj,u,v,fnij/dA,fx/dA,fy/dA,fvalue1,fvalue2,value);
					}
//
// --- force and moment
//
   				Y [0] = Y[0] + fx;
					Y [1] = Y[1] + fy;
					Y [2] = Y[2] + fy*xi - fx*yj;
				}
         }
         if ( draw3d && dataexchange ) fprintf ( DiscreteOutput,"\n" );
      }
   }

   Y [3] = fxst;
   Y [4] = fxsl;
   Y [5] = fyst;
   Y [6] = fysl;

   //if ( vsy > 0.0 ) Y [2] = Y [2]*MzScaleP;
   //else             Y [2] = Y [2]*MzScaleN;
}
void CDiscreteFormula::DiscreteFormula_Terminate ( )
//----------------------------------------------------------------------
// --- terminate formula
//----------------------------------------------------------------------
{
   if ( draw3d == 1 && dataexchange )
   {
      fflush ( DiscreteOutput );
      fclose ( DiscreteOutput );
   }
   free ( bmaxj   );
   free ( bminj   );
   free ( hi      );
   free ( muslide );
   free ( vslide  );

   free_mat ( ni+1,Fzscale );
}
double CDiscreteFormula::DiscreteFormula_Pressure ( const double xi, const double yj )
//--------------------------------------------------------------------------------------------
// --- normal force distribution over rectangular contact patch
//--------------------------------------------------------------------------------------------
{
   double value1,value2;

   value1 = ( xi/h )*( xi/h );
   value2 = ( yj/b )*( yj/b );
   switch ( pressure )
   {
      case 1:
      {
         return ( fnij = sigma0*( 1.0 - value1*value1 )/( valij ) );
      }
      case 2:
      {
         return ( fnij = sigma1*( 1.0 - value1*value1*value1*value1 )*( 1.0 + value2 )/( valij ) );
      }
      case 3:
      {
         return ( fnij = sigma2*( 1.0 - value1*value1*value1*value1 )*( 1.0 + 2.0*value2 - 3.0*value2*value2*value2*value2 )/( valij ) );
      }
   }
   return ( 0.0 );
}

void CDiscreteFormula::DiscreteFormula_Area_Lookup () 
//----------------------------------------------------------------------
// --- get bmax(j) and bmin(j) as well as h(i)
//----------------------------------------------------------------------
{
   int    j,k,njj;
   double yj,hlocal,ylocal,hmax,hmin,bmax,bmin;
   double value1,value2,value3,value4,dnjj;

	cga    = cos( gamma );
   sga    = sin( gamma );
   Ryp    = pow( Ry,(int) ny );
	Ryp1   = sqrt( Ryp );
   dez    = R-Fz/cR;
	value1 = pow( cga, (int) (ny+2) );
//
// --- search for width (ylocal <= 200 [mm]) and length with 100 points
//
	ylocal = 200.0;
	hlocal =   0.0;
	k      =     0;
	njj    =   100;
	dnjj   = (double) (njj-1);
//
// --- check length and width
//
	for ( j=0; j<njj; j++ )
	{
		yj     = -ylocal + 2.0*ylocal*((double) j)/dnjj;
      value2 = yj*cga*sga;
      value3 = value2*( value2 + 2.0*dez ) - R*R*cga*cga + dez*dez;
      value4 = -Ryp*value3 - pow( yj, (int) ny ) *R*R * value1;

		if ( value4 > 0.0 ) 
		{
			hlocal = (sqrt(value4) / ( Ryp1*cga ));
			if ( k == 0 )
			{
				hmax  = hlocal;
				hmin  =-hlocal;
				bmax  = yj;
				bmin  = yj;
				k     = 1;
			}
			else
			{
				if ( hlocal > hmax ) hmax = hlocal;
				if (-hlocal < hmin ) hmin =-hlocal;
				if ( yj     > bmax ) bmax = yj;
				if ( yj     < bmin ) bmin = yj;
			}
		}
	}
//
// --- dimension of potential contact area
//
	h = mymax( fabs( hmax ),fabs( hmin ) );
	b = mymax( fabs( bmax ),fabs( bmin ) );
}
void CDiscreteFormula::DiscreteFormula_Area ()
//----------------------------------------------------------------------
// --- get contact area shape information and pressure scale
//----------------------------------------------------------------------
{
   int i,j;
   double xi,yj;
   double value1,value2,value3,Gij;
//
// --- identify contact area by formulas
//
   cga = cos( gamma );
   sga = sin( gamma );
   Ryp = pow( Ry, (int) ny );
   dez = R-Fz/cR;
//
// --- formula for length hi[j] on potential contact area
//
   for ( j=0; j<=nj; j++ )
   {
      yj      = b - dyf * ( double ) j;
      value2  = yj*cga*sga;
      value3  = value2*( value2 + 2.0*dez ) - R*R*cga*cga + dez*dez;
      value3  = -Ryp*value3 - pow( yj, (int) ny )*R*R*pow( cga,(int) (ny+2) );

      if ( value3 < 0.0 ) hi[j] = 0.0;
      else                hi[j] = sqrt( value3 )/( sqrt( Ryp )*cga );
   }
//
// --- identify contact area width by walking through the grid
//
   for ( i=0; i<=ni; i++ )
   {
      xi = h - dxf * ( double ) i;
		bmaxj[i] =-b;
		bminj[i] = b;
		for ( j=0; j<=nj; j++ )
      {
         yj = b - dyf * ( double ) j;
//
// --- check Gij - boundary
//
         value1 = pow( yj*cga/Ry,(int) ny );
         value2 = yj*sga - sqrt( R*R*( 1.0 -value1 ) -xi*xi );
         Gij    = cga*value2 + dez;
//
// --- new member if inside
//
         if ( Gij <= 0.0 )
         {
            if ( yj > bmaxj[i] ) bmaxj[i] = yj;
            if ( yj < bminj[i] ) bminj[i] = yj;
         }
         Fzscale[i][j] = Gij;
      }
   }
//
// --- get scale value for normal force distribution
//
   sigma3 = 0.0;
	ninside = 0;
   for ( j=0; j<=nj; j++ )
   {
      if ( hi[j] > 0.0 )
      {
         yj = b - dyf * ( double ) j;
         for ( i=0; i<=ni; i++ )
         {
            if ( Fzscale[i][j] <= 0.0 )
            {
               xi     = h - dxf * ( double ) i;
               sigma3 = sigma3 + DiscreteFormula_GetFz ( xi,yj,i,j );
					Area   = Area + dxf*dyf;
					ninside++;
            }
         }
      }
   }
}
void CDiscreteFormula::calc_loaded_radius_AS_LR_Speed ( const double Fz, const double Fy, const double VT )
//----------------------------------------------------------------------------
// --- extended contact area definition: loaded rolling radius calculation
//----------------------------------------------------------------------------
{
	double velo,cam,pres,Fzlocal;

	velo = 0.001*VT;
	cam  = gamma;
	pres = pin*1.0e+06;

	LR_calc.dv = Parameter_LR.av*pow(velo,Parameter_LR.pv);
	LR_calc.dz = Fz/(Parameter_LR.kz*pres+Parameter_LR.p0);
	LR_calc.dy = Fy/(pres*(Parameter_LR.ky+Parameter_LR.kc*cam*MySign(Fy)));
	LR_calc.dr = 0.5*(LR_calc.dy/Parameter_LR.R0)*(LR_calc.dy/Parameter_LR.R0);
	LR_calc.dc = Parameter_LR.ac*cam*cam*(Parameter_LR.az-LR_calc.dz);
	LR_calc.cz = Parameter_LR.kz*pres+Parameter_LR.p0;
	LR_calc.LR = (Parameter_LR.R0 + LR_calc.dv - LR_calc.dz - LR_calc.dr + LR_calc.dc);

	Fzlocal = 0.0;

	LR_calc_unloaded.dv = Parameter_LR.av*pow(velo,Parameter_LR.pv);
	LR_calc_unloaded.dz = Fzlocal/(Parameter_LR.kz*pres+Parameter_LR.p0);
	LR_calc_unloaded.dy = Fy/(pres*(Parameter_LR.ky+Parameter_LR.kc*cam*MySign(Fy)));
	LR_calc_unloaded.dr = 0.5*(LR_calc_unloaded.dy/Parameter_LR.R0)*(LR_calc_unloaded.dy/Parameter_LR.R0);
	LR_calc_unloaded.dc = Parameter_LR.ac*cam*cam*(Parameter_LR.az-LR_calc_unloaded.dz);
	LR_calc_unloaded.cz = Parameter_LR.kz*pres+Parameter_LR.p0;
	LR_calc_unloaded.LR = (Parameter_LR.R0 + LR_calc_unloaded.dv - LR_calc_unloaded.dz - LR_calc_unloaded.dr + LR_calc_unloaded.dc);
}
void CDiscreteFormula::DiscreteFormula_Area_Lookup_Ex (const double Fz, const double Fy, const double VT )
//------------------------------------------------------------------------------------------------------------
// --- get extended contact area shape information: max width and amx length
//------------------------------------------------------------------------------------------------------------
{
	int    i,k=0;
	double R_y,n_y,par_gamma_comp,par_gamma_sin,par_gamma_cos,gamma_comp;
	double par1,par2,par_offset,ky_stat,cosgamma,sc_y,yj,LR,RAS;
	double upper,lower,par_stiffness,par_lateral;
	double hlocal,hmax,hmin,bmax,bmin;

	calc_loaded_radius_AS_LR_Speed(Fz,Fy,VT);

	R_y             = Par.R_y;
	n_y             = Par.ny;
	par_gamma_comp  = Par.gamma_comp;
	par_gamma_sin   = Par.gamma_sin;
	par_gamma_cos   = Par.gamma_cos;
	par1            = Par.p1;
	par2            = Par.p2;
	par_offset      = Par.offset;
	ky_stat         = Par.ky_stat; 

	gamma_comp = ((Fy/par_gamma_comp)*pi/180 + gamma);  
	cosgamma   = cos(gamma_comp*par_gamma_cos);          
   RAS        = LR_calc_unloaded.LR;
   LR         = LR_calc.LR;

	for (i=0; i<100; i++ )
	{
		yj    = -0.2 + 0.4 * ( (double) i ) / 99.0;
		sc_y  = yj/2.0 * sin(gamma_comp * par_gamma_sin);    
		upper = (-pow(R_y,n_y) *(sc_y *(sc_y + 2.0* LR) - (RAS*RAS) * (cosgamma*cosgamma)+ LR*LR) - pow(yj,n_y) * (RAS*RAS) * pow(cosgamma,(n_y+2)));
		lower = (sqrt(pow(R_y,n_y)) * cosgamma);

		if ( upper > 0.0 ) 
		{
			hlocal        = sqrt(upper)/lower;	 
			par_stiffness = hlocal*hlocal * par1 *(1-cos(yj/R_y*pi));
			par_lateral   = Fz * par2* (yj -(Fy/ky_stat))*(yj -(Fy/ky_stat));
			hlocal        = hlocal * (par_offset + par_stiffness + par_lateral);

			if ( k == 0 )
			{
				hmax  = hlocal;
				hmin  =-hlocal;
				bmax  = yj;
				bmin  =-yj;
				k     = 1;
			}
			else
			{
				if ( hlocal > hmax ) hmax = hlocal;
				if (-hlocal < hmin ) hmin =-hlocal;
				if ( yj     > bmax ) bmax = yj;
				if ( yj     < bmin ) bmin = yj;
			}
		}
	}
//
// --- dimension of potential contact area
//
	h = 1000.0 * mymax( fabs( hmax ),fabs( hmin ) );
	b = 1000.0 * mymax( fabs( bmax ),fabs( bmin ) );
}
void CDiscreteFormula::DiscreteFormula_Area_Ex ( const double Fz, const double Fy )
//------------------------------------------------------------------------------------------------------------
// --- get extended contact area shape information and pressure scale
//------------------------------------------------------------------------------------------------------------
{
	int    i,j,k=0;
	double R_y,n_y,par_gamma_comp,par_gamma_sin,par_gamma_cos,gamma_comp;
	double par1,par2,par_offset,ky_stat,cosgamma,sc_y,LR,RAS;
	double upper,lower,par_stiffness,par_lateral;
	double hlocal,xi,yj,value;

	R_y             = Par.R_y;
	n_y             = Par.ny;
	par_gamma_comp  = Par.gamma_comp;
	par_gamma_sin   = Par.gamma_sin;
	par_gamma_cos   = Par.gamma_cos;
	par1            = Par.p1;
	par2            = Par.p2;
	par_offset      = Par.offset;
	ky_stat         = Par.ky_stat; 

	gamma_comp = ((Fy/par_gamma_comp)*pi/180 + gamma);  
	cosgamma   = cos(gamma_comp*par_gamma_cos);          
   RAS        = LR_calc_unloaded.LR;
   LR         = LR_calc.LR;

	for (j=0; j<=nj; j++ )
	{
		yj    = 0.001*(b - dyf * ( double ) j);
		sc_y  = yj/2.0 * sin(gamma_comp * par_gamma_sin);   	 
		upper = (-pow(R_y,n_y) *(sc_y *(sc_y + 2.0* LR) - (RAS*RAS) * (cosgamma*cosgamma)+ LR*LR) - pow(yj,n_y) * (RAS*RAS) * pow(cosgamma,(n_y+2)));
		lower = (sqrt(pow(R_y,n_y)) * cosgamma);

		if ( upper <= 0.0 ) 
		{
			hi[j] = 0.0;
         for ( i=0; i<=ni; i++ ) Fzscale[i][j] = 1.0;
		}
      else                 
		{
			hlocal        = sqrt(upper)/lower;	 
			par_stiffness = hlocal*hlocal * par1 *(1-cos(yj/R_y*pi));
			par_lateral   = Fz * par2* (yj -(Fy/ky_stat))*(yj -(Fy/ky_stat));
			hlocal        = hlocal * (par_offset + par_stiffness + par_lateral);
			hi[j]         = 1000.0*hlocal;

			for ( i=0; i<=ni; i++ )
			{
				Fzscale[i][j] = 1.0;
				if ( i == 0 )
		 		{
					bmaxj[i] =-b;
					bminj[i] = b;
				}
				xi = h - dxf * ( double ) i;
				if ( fabs(xi) <= hi[j] ) 
				{
					Fzscale[i][j] = -1.0;
					value = 1000.0*yj;
					if ( value > bmaxj[i] ) bmaxj[i] = value;
	            if ( value < bminj[i] ) bminj[i] = value;
				}
			}
		}
   }
//
// --- get scale value for normal force distribution
//
   sigma3 = 0.0;
	ninside = 0;
   for ( j=0; j<=nj; j++ )
   {
      if ( hi[j] > 0.0 )
      {
         for ( i=0; i<=ni; i++ )
         {
            if ( Fzscale[i][j] < 0.0 )
            {
               yj     = b - dyf * ( double ) j;
               xi     = h - dxf * ( double ) i;
               sigma3 = sigma3 + DiscreteFormula_GetFz ( xi,yj,i,j );
					Area   = Area + dxf*dyf;
					ninside++;
            }
         }
      }
   }
}
double CDiscreteFormula::DiscreteFormula_GetFz ( const double xi, const double yj, const int i, const int j )
//----------------------------------------------------------------------
// --- get Fz(xi,yj) att a point xi and yj
//----------------------------------------------------------------------
{
	double xpow1,xpow2,ypow1,ypow2,ypow3;
   double value1,value3,value4,value5,Fnij;
//
// --- outside contact area ?
//
   if ( Fzscale[i][j] > 0.0 ) return ( 0.0 );

   if ( hi[j] > 0.0 )
   {
      value1 = fabs( bmaxj[i] - bminj[i] );
      if ( value1 > 0.0 )
      {
//
// --- inside contact area !
//
			value3 = xi/hi[j];
			value4 = yj/b;
         xpow1  = pow(value3,(int) dataxpow1);
         xpow2  = pow(value3,(int) dataxpow2);
			ypow1  = pow(value4,(int) dataypow1);
			ypow2  = pow(value4,(int) dataypow2);
			ypow3  = pow(value4,(int) dataypow3); 

			value3 = (1.0 - xpow1)*(1.0 + xpow1) - 0.20*(1.0 - xpow2)*(1.0 + xpow2);
         value4 = (1.0 - ypow1)*(1.0 + ypow1) - 1.00*(1.0 - ypow2)*(1.0 + ypow2); 
			value5 = (1.0 - ypow3*xygamma*sin(gamma));
			Fnij   = value3*(value4+value5)*Fzmag;
//
// --- in the case of input errors
//
			if ( Fnij < 0.0 ) Fnij = 0.0;
         return ( Fnij );
      }
   }
   return ( 0.0 );
}
double CDiscreteFormula::DiscreteFormula_Friction ()
//----------------------------------------------------------------------
// --- get sliding parts of tangential contact forces
//----------------------------------------------------------------------
{
   double vtij,ftij,fuij;
   double mue0Fz,muex1Fz,muey1Fz,cmuexFz,cmueyFz;
//
// --- case depended friction sliding values
//
   if ( vsx > 0.0 )
   {
      mue1x = mue1xp;
      cmuex = cmuexp;
   }
   else
   {
      mue1x = mue1xn;
      cmuex = cmuexn;
   }

   if ( vsy > 0.0 )
   {
      mue1y = mue1yp;
      cmuey = cmueyp;
   }
   else
   {
      mue1y = mue1yn;
      cmuey = cmueyn;
   }
//
// --- calculate magnitudes
//
   vtij = sqrt( vsx*vsx + vsy*vsy );
   ftij = sqrt( fx*fx + fy*fy );
   fuij = sqrt( u*u   + v*v   );

   if ( ftij < 1.0e-10 ) ftij = 1.0;
   if ( vtij < 1.0e-10 ) vtij = 1.0;
   if ( fuij < 1.0e-10 ) fuij = 1.0;
//
// --- direction depended friction coefficients
//
	if ( version != 202 )
	{
		mue0Fz  = mue0* ( 1.0 + mueFz*( 1.0 + ( Fzbar-Fz ) ) );
		muex1Fz = mue1x*( 1.0 + mueFz*( 1.0 + ( Fzbar-Fz ) ) );
		muey1Fz = mue1y*( 1.0 + mueFz*( 1.0 + ( Fzbar-Fz ) ) );
		cmuexFz = cmuex*( 1.0 + mueFz*( 1.0 + ( Fzbar-Fz ) ) );
		cmueyFz = cmuey*( 1.0 + mueFz*( 1.0 + ( Fzbar-Fz ) ) );

		muex = mue0Fz + ( muex1Fz-mue0Fz )*( 1.0 - exp( -fabs( vtij ) /cmuexFz ) )*( 1.0 + mueFz*( 1.0 + ( Fzbar-Fz ) ) );
		muey = mue0Fz + ( muey1Fz-mue0Fz )*( 1.0 - exp( -fabs( vtij ) /cmueyFz ) )*( 1.0 + mueFz*( 1.0 + ( Fzbar-Fz ) ) );

	}
	else
	{
		mue0Fz  = mue0* ( 1.0 + mueFz*( ( Fz/Fzbar ) - 1.0 ) );
		muex1Fz = mue1x*( 1.0 + mueFz*( ( Fz/Fzbar ) - 1.0 ) );
		muey1Fz = mue1y*( 1.0 + mueFz*( ( Fz/Fzbar ) - 1.0 ) );
		cmuexFz = cmuex*( 1.0 + mueFz*( ( Fz/Fzbar ) - 1.0 ) );
		cmueyFz = cmuey*( 1.0 + mueFz*( ( Fz/Fzbar ) - 1.0  ) );

		muex = mue0Fz + ( muex1Fz-mue0Fz )*( 1.0 - exp( -fabs( vtij ) /cmuexFz ) )*( 1.0 + mueFz*( ( Fzbar/Fz ) - 1.0 ) );
		muey = mue0Fz + ( muey1Fz-mue0Fz )*( 1.0 - exp( -fabs( vtij ) /cmueyFz ) )*( 1.0 + mueFz*( ( Fzbar/Fz ) - 1.0 ) );
	}
//
// --- pseudo sliding area: different approaches
//
   if ( ftij > mue0*fnij )
   {
      switch ( sliding )
      {
         case 1: // direction of slip velocity
         {
            fx = fnij * muex * vsx / vtij;
            fy = fnij * muey * vsy / vtij;
            return(1.0);
         }
         case 2: // direction of displacement
         {
            fx = fnij * muex * u / fuij;
            fy = fnij * muey * v / fuij;
            return(1.0);
         }
         case 3: // direction of tangential contact force
         {
            fx = fnij * muex * fx / ftij;
            fy = fnij * muey * fy / ftij;
            return(1.0);
         }
      }
   }
	return(0.0);
}
double CDiscreteFormula::DiscreteFormula_Friction_FB ()
//----------------------------------------------------------------------
// --- picewise linear friction function
//----------------------------------------------------------------------
{
   int    i;
   double value1,value2,vtij,mue,mue0Fz,ftij;
//
// --- local stress depended friction value
//
   value1 = fnij / ( dxf*dyf );
   value2 = 0.5e+00 * ( frica0 + exp( -frica1*value1 ) );
//
// --- global stress depended friction value
//
	mue0Fz = FrictSca * mue0 * value2*( 1.0 + mueFz*( ( Fz/Fzbar ) - 1.0 ) );

   ftij   = sqrt( fx*fx + fy*fy );
//
// --- test for sliding
//
   if ( ftij > mue0Fz*fnij )
   {
      vtij = sqrt( Vmuexy*Vmuexy*vsx*vsx + vsy*vsy );
      mue  = muslide[islide-1];
      for ( i=1; i<islide; i++ )
      {
         if ( vtij < vslide[i] )
         {
            mue = muslide[i-1] + ( muslide[i] - muslide[i-1] )  /  ( vslide[i] - vslide[i-1] )*( vtij   - vslide[i-1] );
            break;
         }
      }
//
// --- global stress depended friction value
//
		mue    = FrictSca *  mue  * value2*( 1.0 + mueFz*( ( Fz/Fzbar ) - 1.0 ) );

      ftij = sqrt( fx*fx + fy*fy );
      fx = fnij * mue * muexy * fx / ftij;
      fy = fnij * mue         * fy / ftij;
      fxsl += fx;
      fysl += fy;
		return(1.0);
   }
   else
   {
      fxst += fx;
      fyst += fy;
		return(0.0);
   }
}
//-----------------------------------------------------------------------------------------------------
double **CDiscreteFormula::alloc_mat ( int ir, int ic, double fac )
//-----------------------------------------------------------------------------------------------------
// allocate space for irow times icolumn matrix and return pointer
//-----------------------------------------------------------------------------------------------------
{
   int i,j;
   double **p;

   p = ( double ** ) malloc ( ir * sizeof ( double * ) );

   if ( p == NULL )
   {
      printf ( " No space for allocate left \n" );
      exit ( 0 );
   }

   for ( i=0; i<ir; i++ )
   {
      p[i] = ( double * ) malloc ( ic * sizeof ( double ) );

      if ( p[i] == NULL )
      {
         printf ( " No space for allocate left \n" );
         exit ( 0 );
      }
   }

   for ( i=0; i<ir; i++ )
   {
      for ( j=0; j<ic; j++ )
      {
         p[i][j] = fac;
      }
   }
   return ( p );
}
//-----------------------------------------------------------------------------------------------------
void CDiscreteFormula::free_mat ( int ir, double **p )
//-----------------------------------------------------------------------------------------------------
// allocate space for irow times icolumn matrix and return pointer
//-----------------------------------------------------------------------------------------------------
{
   for ( int i=0; i<ir; i++ )
   {
      free( p[i] );
   }
   free( p );
}
//-----------------------------------------------------------------------------------------------------
double CDiscreteFormula::mymax ( const double value1, const double value2 )
//-----------------------------------------------------------------------------------------------------
// return max(x,y)
//-----------------------------------------------------------------------------------------------------
{
   if ( value1 > value2 ) return ( value1 );
   return ( value2 );
}
//-----------------------------------------------------------------------------------------------------
double CDiscreteFormula::mymin ( const double value1, const double value2 )
//-----------------------------------------------------------------------------------------------------
// return min(x,y)
//-----------------------------------------------------------------------------------------------------
{
   if ( value1 < value2 ) return ( value1 );
   return ( value2 );
}