#include "StdAfx.h"
#include "FormulaGUIData.h"

//---------------------------------------------------------------------
CFormulaGUIData::CFormulaGUIData(void)
//---------------------------------------------------------------------
{
	MyRun        = new CRMOD_K_Formula_Opti;
	MyMF         = new CMF5p2;
	design       = (double *) malloc (sizeof(double)*NMODELPARAMETER);
	newinput     = (double *) malloc (sizeof(double)*NMODELPARAMETER);
	baseinput    = (double *) malloc (sizeof(double)*NMODELPARAMETER);
	ValuesLoaded = 0;
	DataLoaded   = 0;
	version      = 201;
	JobTitle     = L" ";
}
//---------------------------------------------------------------------
void CFormulaGUIData::Evaluate()
//---------------------------------------------------------------------
{
	double modeofcall;

	if ( DataLoaded == 0 ) return;
	modeofcall = 1.0;
	target = MyRun->Set_TargetValue(design,&modeofcall); 
}
//---------------------------------------------------------------------
void CFormulaGUIData::EvaluateAll()
//---------------------------------------------------------------------
{
	double modeofcall;

	if ( DataLoaded == 0 ) return;
	modeofcall = -1.0;
	target = MyRun->Set_TargetValue(design,&modeofcall); 
}
//---------------------------------------------------------------------
void CFormulaGUIData::GetData() 
//---------------------------------------------------------------------
{
	MyRun->Get_Changes(baseinput);
}
//---------------------------------------------------------------------
void CFormulaGUIData::LoadData(int mode) 
//---------------------------------------------------------------------
// load measurement or reference data
//---------------------------------------------------------------------
{
   int    index;
	double modeofcall;
   System::Windows::Forms::DialogResult    FileDialog;
	System::Windows::Forms::OpenFileDialog ^ParameterFile;

   if ( ValuesLoaded == 0 ) 
	{
		System::Windows::Forms::MessageBox::Show(" Load parameter and reference first","Hint");
		return;
	}

	if ( mode == 1 )
	{
		// --- file selection for solver file, set the initial values of filedialog
		ParameterFile = gcnew System::Windows::Forms::OpenFileDialog;
		ParameterFile->Filter  =  "target file (*.dat)|*.dat|All files (*.*)|*.*";
		ParameterFile->FileName = "DesignTarget.dat";

		FileDialog = ParameterFile->ShowDialog();

		// --- get the name of filedialog and extract the path and the filename of solver file
		if ( FileDialog == System::Windows::Forms::DialogResult::OK ) 
		{
			// --- get the path to the file and the solver filename
			File2 = ParameterFile->FileName::get();
			File3 = ParameterFile->FileName::get();
			index = File2->LastIndexOf("\\");
			sprintf (MyRun->OptiFileName,"%s",File2);
			File2 = File2->Substring(0,index+1);
			if ( index > 0 ) sprintf (MyRun->WorkDirectory,"%s",File2);
		}
	}
	MyRun->freemem();
	MyRun->Set_AllocState(0);
	MyRun->Set_LoadedState(0);
	MyRun->Get_TargetData();
	modeofcall =  0.0;
	MyRun->Set_TargetValue(design,&modeofcall); 
	modeofcall = -1.0;
	target = MyRun->Set_TargetValue(design,&modeofcall); 
	DataLoaded = 1;
}
//---------------------------------------------------------------------
void CFormulaGUIData::LoadMFData() 
//---------------------------------------------------------------------
{
	int  index;
	char s[512];

   System::Windows::Forms::DialogResult    FileDialog;
	System::Windows::Forms::OpenFileDialog ^ParameterFile;

   // --- file selection for MF file, set the initial values of filedialog
   ParameterFile = gcnew System::Windows::Forms::OpenFileDialog;
   ParameterFile->Filter  =  "MF data file (*.tir)|*.tir|All files (*.*)|*.*";
   ParameterFile->FileName = "Design.tir";

   FileDialog = ParameterFile->ShowDialog();

   // --- get the name of file and extract the path and the filename of MF file
   if ( FileDialog == System::Windows::Forms::DialogResult::OK ) 
   {
		MFName = ParameterFile->FileName::get();
		index  = MFName->LastIndexOf("\\");
		MFName = MFName->Substring(index+1);
	
		ConvertStringToChar(ParameterFile->FileName::get(),s);
		MyMF->get_typarr(s);
		MyMF->read_values();
   }
}
//---------------------------------------------------------------------
bool CFormulaGUIData::LoadOptiMethod()
//---------------------------------------------------------------------
{
   int    index;
   System::Windows::Forms::DialogResult    FileDialog;
	System::Windows::Forms::OpenFileDialog ^MethodFile;

   if ( ValuesLoaded == 0 ) 
	{
		System::Windows::Forms::MessageBox::Show(" Load parameter and reference first","Hint");
		return(false);
	}

   MethodFile = gcnew System::Windows::Forms::OpenFileDialog;
   MethodFile->Filter  =  "optimisation work file (*.m;*.sce)|*.m;*.sce|All files (*.*)|*.*";
   MethodFile->FileName = "";

   MethodFile->InitialDirectory::set(Optipath);
   FileDialog = MethodFile->ShowDialog();

   if ( FileDialog == System::Windows::Forms::DialogResult::OK ) 
   {
		File2 = MethodFile->FileName::get();
      index = File2->LastIndexOf("\\");
      OptiMethodFile = File2->Substring(index+1);
		return (true);
   }
	return(false);
}
//---------------------------------------------------------------------
void CFormulaGUIData::LoadParameter() 
//---------------------------------------------------------------------
// load modal parameter
//---------------------------------------------------------------------
{
	int index;
   System::Windows::Forms::DialogResult    FileDialog;
	System::Windows::Forms::OpenFileDialog ^ParameterFile;

   // --- file selection for solver file, set the initial values of filedialog
   ParameterFile = gcnew System::Windows::Forms::OpenFileDialog;
   ParameterFile->Filter  =  "parameter file (*.dat)|*.dat|All files (*.*)|*.*";
   ParameterFile->FileName = "DesignStart.dat";

   FileDialog = ParameterFile->ShowDialog();

   // --- get the name of filedialog and extract the path and the filename of solver file
   if ( FileDialog == System::Windows::Forms::DialogResult::OK ) 
   {
      // --- get the path to the file and the solver filename
		File1 = ParameterFile->FileName::get();
		File2 = ParameterFile->FileName::get();
		sprintf (MyRun->DataFileName,"%s",File1);
		MyRun->ReadFile_Formula(design);
		MyRun->Set_LoadedState(0);
		Nmodelparameter = MyRun->Get_NModelPar();
		ReReadParameter();
		ValuesLoaded = 1;
		for (int i=0; i<Nmodelparameter; i++)
		{
			baseinput[i] = MyRun->input1[i];
			newinput[i]  = MyRun->input1[i];
		} 
		index = File2->LastIndexOf("\\");
      File2 = File1->Substring(0,index+1);
      if ( index > 0 ) sprintf (MyRun->WorkDirectory,"%s",File2);
   }
}
//---------------------------------------------------------------------
void CFormulaGUIData::SaveParameter()  
//---------------------------------------------------------------------
// save modal parameter
//---------------------------------------------------------------------
{
   System::Windows::Forms::DialogResult    FileDialog;
	System::Windows::Forms::SaveFileDialog ^ParameterFile;
   System::String ^str1;

   // --- file selection for solver file, set the initial values of filedialog
   ParameterFile = gcnew System::Windows::Forms::SaveFileDialog;
   ParameterFile->Filter  =  "parameter file (*.dat)|*.dat|All files (*.*)|*.*";
   ParameterFile->FileName = "DesignStatus.dat";

   FileDialog = ParameterFile->ShowDialog();

   // --- get the name of filedialog and extract the path and the filename of solver file
   if ( FileDialog == System::Windows::Forms::DialogResult::OK ) 
   {
      // --- get the path to the file and the solver filename
		str1 = ParameterFile->FileName::get();
		sprintf (MyRun->DataResultName,"%s",str1);
		sprintf (MyRun->JobTitle,      "%s",JobTitle);
		MyRun->Set_version (version);
   }

	if ( ValuesLoaded ) MyRun->get_variables(design);
}
//---------------------------------------------------------------------
bool CFormulaGUIData::SaveGraphis()  
//---------------------------------------------------------------------
// save modal parameter
//---------------------------------------------------------------------
{
   System::Windows::Forms::DialogResult    FileDialog;
	System::Windows::Forms::SaveFileDialog ^GraphicsFile;

   // --- file selection for solver file, set the initial values of filedialog
   GraphicsFile = gcnew System::Windows::Forms::SaveFileDialog;
   GraphicsFile->Filter  =  "graphics file (*.bmp)|*.bmp|All files (*.*)|*.*";
   GraphicsFile->FileName = "Picture.bmp";

   FileDialog = GraphicsFile->ShowDialog();

   // --- get the name of filedialog and extract the path and the filename of solver file
   if ( FileDialog == System::Windows::Forms::DialogResult::OK ) 
   {
		File4 = GraphicsFile->FileName;
		return (true);
	}
	return (false);
}
//---------------------------------------------------------------------
void CFormulaGUIData::ReReadParameter()  
//---------------------------------------------------------------------
// read model parameter names into strarray
//---------------------------------------------------------------------
{
	FILE *INPUTDATA;
	char s [1024];
	char s1[1024];
	char *s2,*s3;
	int  i,linecount,totalline=0;
//
// --- somebody at home?
//
   INPUTDATA = fopen ( MyRun->DataFileName,"r+" );
//
// --- parse input file
//
	fgets ( s,1024,INPUTDATA );
	if ( ( s2 = strstr (s,"version")) != NULL  ) sscanf (s2+7,"%d",&version);           else version = 201;
	if ( ( s3 = strstr (s,"title"))   != NULL  )	JobTitle = ConvertCharToString(s3+5);  else JobTitle = ConvertCharToString("NewFile");										       

   if ( strstr ( s,"[RMFpar1]" ) )
   {
		linecount = MyRun->get_integevalue ( s );
      for ( i=0; i<linecount; i++ )
      {
         fgets ( s,1024,INPUTDATA );
         MyRun->get_charvalue ( s,s1 );
			strarray[totalline++] = ConvertCharToString(s1);
      }
   }

   fgets ( s,1024,INPUTDATA );
   if ( strstr ( s,"[RMFpar2]" ) )
   {
		linecount = MyRun->get_integevalue ( s );
      for ( i=0; i<linecount; i++ )
      {
         fgets ( s,1024,INPUTDATA );
         MyRun->get_charvalue ( s,s1 );
			strarray[totalline++] = ConvertCharToString(s1);
      }
   }

   fgets ( s,1024,INPUTDATA );
   if ( strstr ( s,"[RMFpar3]" ) )
   {
		linecount = MyRun->get_integevalue ( s );
      for ( i=0; i<linecount; i++ )
      {
         fgets ( s,1024,INPUTDATA );
         MyRun->get_charvalue ( s,s1 );
			strarray[totalline++] = ConvertCharToString(s1);
      }
   }

   fgets ( s,1024,INPUTDATA );
   if ( strstr ( s,"[RMFpar4]" ) )
   {
 	  linecount = MyRun->get_integevalue ( s );
     for ( i=0; i<linecount; i++ )
      {
         fgets ( s,1024,INPUTDATA );
         MyRun->get_charvalue ( s,s1 );
			strarray[totalline++] = ConvertCharToString(s1);
      }
   }


   fgets ( s,1024,INPUTDATA );
   if ( strstr ( s,"[RMFpar5]" ) )
   {
 	  linecount = MyRun->get_integevalue ( s );
     for ( i=0; i<linecount; i++ )
      {
         fgets ( s,1024,INPUTDATA );
         MyRun->get_charvalue ( s,s1 );
			strarray[totalline++] = ConvertCharToString(s1);
     }
   }

	if ( version >= 203 ) 
	{
		fgets ( s,1024,INPUTDATA );
		if ( strstr ( s,"[RMFpar6]" ) )
		{
 		  linecount = MyRun->get_integevalue ( s );
		  for ( i=0; i<linecount; i++ )
			{
				fgets ( s,1024,INPUTDATA );
				MyRun->get_charvalue ( s,s1 );
				strarray[totalline++] = ConvertCharToString(s1);
		  }
		}
	}
//
// --- read model type
//
   fgets ( s,1024,INPUTDATA );
   if ( strstr ( s,"[RMFmodel]" ) )
   {
      fgets ( s,1024,INPUTDATA );
      MyRun->get_charvalue ( s,s1 );
		strarray[totalline++] = ConvertCharToString(s1);
   }

   fclose ( INPUTDATA );
}
// -------------------------------------------------------------------------------------------------------------
String ^CFormulaGUIData::ConvertCharToString(char *charstring)
// -------------------------------------------------------------------------------------------------------------
// --- convert the String s to a char* 
// -----------------------------------------------------------
{
	String ^s;
	s = Char::ToString(charstring[0]);
	for (int j=1; j<(int) strlen(charstring); j++) s += Char::ToString(charstring[j]); 
	return (s);
}
void CFormulaGUIData::ConvertStringToChar(String ^s, char *charstring)
// --------------------------------------------------------------------------------------
// --- convert the String s to a char* 
// --------------------------------------------------------------------------------------
{
//
// --- get the pointer of string s to a const wchar "wchar"
//
	cli::pin_ptr<const wchar_t> wchar = PtrToStringChars(s);
//
// --- get the length of string s
//
   size_t convertedChars = 0;
   size_t sizeInBytes    = ((s->Length + 1) * 2);
   errno_t err = 0;
//
// --- convert the wchar to char*
//
   err = wcstombs_s(&convertedChars,charstring,sizeInBytes,wchar,sizeInBytes);

   return;
}